@@ -1,4 +1,4 @@
-#! /usr/bin/perl
+#!/usr/bin/perl
 
 # debchange: update the debian changelog using your favorite visual editor
 # For options, see the usage message below.
@@ -39,10 +39,14 @@ use Getopt::Long qw(:config gnu_getopt);
 use File::Copy;
 use File::Basename;
 use Cwd;
-use Dpkg::Compression;
 use Dpkg::Vendor qw(get_current_vendor);
-use lib '/usr/share/devscripts';
+use Dpkg::Changelog::Parse;
+BEGIN { push @INC, '/usr/share/devscripts'; }
+use Devscripts::Compression;
 use Devscripts::Debbugs;
+use POSIX qw(locale_h strftime);
+
+setlocale(LC_TIME, "C"); # so that strftime is locale independent
 
 # Predeclare functions
 sub fatal($);
@@ -54,6 +58,8 @@ my $modified_conf_msg;
 my %env;
 my $CHGLINE;  # used by the format O section at the end
 
+my $compression_re = compression_get_file_extension_regex();
+
 my $lpdc_broken;
 
 sub have_lpdc {
@@ -128,7 +134,7 @@ Options:
          Add a new changelog entry with version number specified
   -e, --edit
          Don't change version number or add a new changelog entry, just
-         update the changelog's stamp and open up an editor
+         opens an editor
   -r, --release
          Update the changelog timestamp. If the distribution is set to
          "UNRELEASED", change it to unstable (or another distribution as
@@ -456,7 +462,7 @@ if (defined $opt_u) {
 
 # See if we're Debian, Ubuntu or someone else, if we can
 my $vendor;
-if (not $opt_vendor eq '') {
+if (defined $opt_vendor && $opt_vendor) {
     $vendor = $opt_vendor;
 } else {
     if (defined $opt_D) {
@@ -611,7 +617,7 @@ else {  # $opt_create
 #####
 
 # Find the current version number etc.
-my %changelog;
+my $changelog;
 my $PACKAGE = 'PACKAGE';
 my $VERSION = 'VERSION';
 my $MAINTAINER = 'MAINTAINER';
@@ -625,79 +631,42 @@ my $CHANGES = '';
 my $CL_URGENCY = '';
 
 if (! $opt_create || ($opt_create && $opt_news)) {
-    if (! $opt_create) {
-	open PARSED, qq[dpkg-parsechangelog -l"$changelog_path" | ]
-	    or fatal "Cannot execute dpkg-parsechangelog: $!";
-    } elsif ($opt_create && $opt_news) {
-	open PARSED, qq[dpkg-parsechangelog | ]
-	    or fatal "Cannot execute dpkg-parsechangelog: $!";
-    } else {
-	fatal "This can't happen: what am I parsing?";
-    }
-
-    my $last;
-    while (<PARSED>) {
-	chomp;
-	if (/^(\S+):\s(.+?)\s*$/) { $changelog{$1}=$2; $last=$1; }
-	elsif (/^(\S+):\s$/) { $changelog{$1}=''; $last=$1; }
-	elsif (/^\s\.$/) { $changelog{$last}.="\n"; }
-	elsif (/^\s(.+)$/) { $changelog{$last}.="$1\n"; }
-	else {
-	    fatal "Don't understand dpkg-parsechangelog output: $_";
-	}
-    }
-
-    close PARSED
-	or fatal "Problem executing dpkg-parsechangelog: $!";
-    if ($?) { fatal "dpkg-parsechangelog failed!"; }
+    my $file = $opt_create ? 'debian/changelog' : $changelog_path;
+    $changelog = Dpkg::Changelog::Parse::changelog_parse(file => $file);
 
     # Now we've read the changelog, set some variables and then
     # let's check the directory name is sensible
     fatal "No package name in changelog!"
-	unless exists $changelog{'Source'};
-    $PACKAGE = $changelog{'Source'};
+	unless exists $changelog->{Source};
+    $PACKAGE = $changelog->{Source};
     fatal "No version number in changelog!"
-	unless exists $changelog{'Version'};
-    $VERSION=$changelog{'Version'};
+	unless exists $changelog->{Version};
+    $VERSION=$changelog->{Version};
     fatal "No maintainer in changelog!"
-	unless exists $changelog{'Maintainer'};
-    ($MAINTAINER,$EMAIL) = ($changelog{'Maintainer'} =~ /^([^<]+) <(.*)>/);
+	unless exists $changelog->{Maintainer};
+    $changelog->{Maintainer} = decode_utf8($changelog->{Maintainer});
+    ($MAINTAINER,$EMAIL) = ($changelog->{Maintainer} =~ /^([^<]+) <(.*)>/);
     fatal "No distribution in changelog!"
-	unless exists $changelog{'Distribution'};
+	unless exists $changelog->{Distribution};
     if ($vendor eq 'Ubuntu') {
 	# In Ubuntu the development release regularly changes, don't just copy
 	# the previous name.
 	$DISTRIBUTION=get_ubuntu_devel_distro();
     } else {
-	$DISTRIBUTION=$changelog{'Distribution'};
+	$DISTRIBUTION=$changelog->{Distribution};
     }
     fatal "No changes in changelog!"
-	unless exists $changelog{'Changes'};
+	unless exists $changelog->{Changes};
 
     # Find the current package version
     if ($opt_news) {
 	my $found_version = 0;
 	my $found_urgency = 0;
-	open PARSED, qq[dpkg-parsechangelog -l"$real_changelog_path" | ]
-	    or fatal "Cannot execute dpkg-parsechangelog: $!";
-	while (<PARSED>) {
-	    chomp;
-	    if (m%^Version:\s+(\S+)$%) {
-		$VERSION = $1;
-		$VERSION =~ s/~$//;
-		$found_version = 1;
-		last if $found_urgency;
-	    } elsif (m%^Urgency:\s+(\S+)(\s|$)%) {
-		$CL_URGENCY = $1;
-		$found_urgency = 1;
-		last if $found_version;
-	    } elsif (m%^$%) {
-		last;
-	    }
-	}
-	close PARSED
-	    or fatal "Problem executing dpkg-parsechangelog: $!";
-	if ($?) { fatal "dpkg-parsechangelog failed!"; }
+	my $clog = Dpkg::Changelog::Parse::changelog_parse(file => $real_changelog_path);
+	$VERSION = $clog->{Version};
+	$VERSION =~ s/~$//;
+
+	$CL_URGENCY = $clog->{Urgency};
     }
 
     # Is the directory name acceptable?
@@ -871,7 +840,7 @@ if ($opt_auto_nmu eq 'yes' and ! $opt_v and ! $opt_l and ! $opt_s and
     ! $opt_create and ! $opt_a_passed and ! $opt_r and ! $opt_e and
     $vendor ne 'Ubuntu' and $vendor ne 'Tanglu' and
     ! ($opt_release_heuristic eq 'changelog' and
-       $changelog{'Distribution'} eq 'UNRELEASED' and ! $opt_i_passed)) {
+       $changelog->{Distribution} eq 'UNRELEASED' and ! $opt_i_passed)) {
 
     if (-f 'debian/control') {
 	if (have_lpdc()) {
@@ -885,7 +854,7 @@ if ($opt_auto_nmu eq 'yes' and ! $opt_v and ! $opt_l and ! $opt_s and
 
 	    if ($maintainer !~ m/<packages\@qa\.debian\.org>/ and
 		! grep { $_ eq $packager } ($maintainer, @uploaders) and
-		$packager ne $changelog{'Maintainer'} and ! $opt_team) {
+		$packager ne $changelog->{Maintainer} and ! $opt_team) {
 		$opt_n=1;
 		$opt_a=0;
 	    }
@@ -989,11 +958,14 @@ if (@ARGV and ! $TEXT) {
 }
 
 # Get the date
-my $date_cmd = ($opt_tz ? "TZ=$opt_tz " : "") . "date -R";
-chomp(my $DATE=`$date_cmd`);
+my $DATE;
+{
+    local $ENV{TZ} = $opt_tz if $opt_tz;
+    $DATE = strftime "%a, %d %b %Y %T %z", localtime();
+}
 
 if ($opt_news && !$opt_i && !$opt_a) {
-    if ($VERSION eq $changelog{'Version'} && !$opt_v && !$opt_l) {
+    if ($VERSION eq $changelog->{Version} && !$opt_v && !$opt_l) {
 	$opt_a = 1;
     } else {
 	$opt_i = 1;
@@ -1033,7 +1005,7 @@ if (! $opt_i && ! $opt_v && ! $opt_d && ! $opt_a && ! $opt_e && ! $opt_r &&
 	}
     }
     elsif ($opt_release_heuristic eq 'changelog') {
-	if ($changelog{'Distribution'} eq 'UNRELEASED') {
+	if ($changelog->{Distribution} eq 'UNRELEASED') {
 		$opt_a = 1;
 	}
 	elsif ($EMPTY_TEXT==1) {
@@ -1052,7 +1024,7 @@ unless ($opt_create) {
     open S, $changelog_path or fatal "Cannot open existing $changelog_path: $!";
 
     # Read the first stanza from the changelog file
-    # We do this directly rather than reusing $changelog{'Changes'}
+    # We do this directly rather than reusing $changelog->{Changes}
     # so that we have the verbatim changes rather than a (albeit very
     # slightly) reformatted version. See Debian bug #452806
 
@@ -1081,7 +1053,7 @@ my $merge=0;
 
 if (($opt_i || $opt_n || $opt_bn || $opt_qa || $opt_R || $opt_s || $opt_team ||
      $opt_bpo || $opt_l || $opt_v || $opt_d ||
-    ($opt_news && $VERSION ne $changelog{'Version'})) && ! $opt_create) {
+    ($opt_news && $VERSION ne $changelog->{Version})) && ! $opt_create) {
 
     $optionsok=1;
 
@@ -1177,7 +1149,7 @@ if (($opt_i || $opt_n || $opt_bn || $opt_qa || $opt_R || $opt_s || $opt_team ||
 	             not $start =~ /build/ and not $start =~ /ubuntu/) {
 		$end .= "build1";
 	    } elsif ($opt_R and $vendor eq 'Tanglu' and
-	             not $start =~ /b/ and not $start =~ /tanglu/) {
+	             not "$start$end" =~ /(b\d+)$/ and not $start =~ /tanglu/) {
 		$end .= "b1";
 	    } elsif ($opt_bpo and not $start =~ /~bpo[0-9]+\+$/) {
 		# If it's not already a backport make it so
@@ -1202,10 +1174,10 @@ if (($opt_i || $opt_n || $opt_bn || $opt_qa || $opt_R || $opt_s || $opt_team ||
 		} elsif (($opt_i or $opt_s) and $vendor eq 'Tanglu' and
 		     $start !~ /(tanglu)(\d+\.)*$/ and not $opt_U) {
 
-		    if ($start =~ /b/) {
-			# Drop bX suffix in favor of tanglu1
-			$start =~ s/b//;
-			$end = "";
+		    if ("$start$end" =~ /(b\d+)$/) {
+				# Drop bX suffix in favor of tanglu1
+				$start =~ s/b$//;
+				$end = "";
 		    }
 		    $end .= "tanglu1";
 		} else {
@@ -1222,7 +1194,7 @@ if (($opt_i || $opt_n || $opt_bn || $opt_qa || $opt_R || $opt_s || $opt_team ||
 			$bpo_dist = $bpo_dists{$previous_dist} . '-backports';
 		    } else {
 			# Fallback to using the previous distribution
-			$bpo_dist = $changelog{'Distribution'};
+			$bpo_dist = $changelog->{Distribution};
 		    }
 		}
 
@@ -1268,16 +1240,18 @@ if (($opt_i || $opt_n || $opt_bn || $opt_qa || $opt_R || $opt_s || $opt_team ||
     if ($opt_news) {
 	$urgency ||= $CL_URGENCY;
     }
-    $urgency ||= 'low';
+    $urgency ||= 'medium';
 
     if (($opt_v or $opt_i or $opt_l or $opt_d) and
-	$opt_release_heuristic eq "changelog" and
-	$changelog{'Distribution'} eq "UNRELEASED" and
-	$distribution eq "UNRELEASED") {
+	$opt_release_heuristic eq 'changelog' and
+	$changelog->{Distribution} eq 'UNRELEASED' and
+	$distribution eq 'UNRELEASED') {
 
 	$merge = 1;
     } else {
-	print O "$PACKAGE ($NEW_VERSION) $distribution; urgency=$urgency\n\n";
+	print O "$PACKAGE ($NEW_VERSION) $distribution; urgency=$urgency";
+	print O ", binary-only=yes" if ($opt_bn);
+	print O "\n\n";
 	if ($opt_n && ! $opt_news) {
 	    print O "  * Non-maintainer upload.\n";
 	    $line = 1;
@@ -1491,7 +1465,7 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
     }
 
     if ($opt_t && $opt_a) {
-	print O "\n -- $changelog{'Maintainer'}  $changelog{'Date'}\n";
+	print O "\n -- $changelog->{Maintainer}  $changelog->{Date}\n";
     } else {
 	print O "\n -- $MAINTAINER <$EMAIL>  $DATE\n";
     }
@@ -1512,7 +1486,7 @@ elsif ($opt_e && ! $opt_create) {
     print O $CHANGES;
 
     if ($opt_t) {
-	print O "\n -- $changelog{'Maintainer'}  $changelog{'Date'}\n";
+	print O "\n -- $changelog->{Maintainer}  $changelog->{Date}\n";
     } else {
 	print O "\n -- $MAINTAINER <$EMAIL>  $DATE\n";
     }
@@ -1537,7 +1511,7 @@ elsif ($opt_create) {
     if ($opt_news) {
 	$urgency ||= $CL_URGENCY;
     }
-    $urgency ||= 'low';
+    $urgency ||= 'medium';
     print O "$PACKAGE ($VERSION) $DISTRIBUTION; urgency=$urgency\n\n";
 
     if (@closes_text or $TEXT) {
@@ -1634,7 +1608,7 @@ if ((basename(cwd()) =~ m%^\Q$PACKAGE\E-\Q$UVERSION\E$%) &&
 	}
 	# And check whether a new orig tarball exists
 	my @origs = glob("../$PACKAGE\_$new_uversion.*");
-	my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.$compression_re_file_ext$/ } @origs;
+	my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.$compression_re$/ } @origs;
 	if ($num_origs == 0) {
 	    warn "$progname warning: no orig tarball found for the new version.\n";
 	}
