@@ -1,7 +1,7 @@
 #!/usr/bin/perl -w
 # vim:sw=4:sta:
 
-# Copyright (C) 2006-2010 Christoph Berg <myon@debian.org>
+# Copyright (C) 2006-2013 Christoph Berg <myon@debian.org>
 #           (C) 2010 Uli Martens <uli@youam.net>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -47,16 +47,15 @@ EOT
 }
 
 my %url_map = (
-    'debian' => "http://qa.debian.org/madison.php",
-    'qa' => "http://qa.debian.org/madison.php",
-    'myon' => "http://qa.debian.org/~myon/madison.php",
-    'bpo' => "http://backports.debian.org/cgi-bin/madison.cgi",
+    'debian' => "https://qa.debian.org/madison.php",
+    'qa' => "https://qa.debian.org/madison.php",
+    'myon' => "https://qa.debian.org/~myon/madison.php",
     'debug' => "http://debug.debian.net/cgi-bin/madison.cgi",
     'ubuntu' => "http://people.canonical.com/~ubuntu-archive/madison.cgi",
-    'udd' => 'http://qa.debian.org/cgi-bin/madison.cgi',
-    'new' => 'http://qa.debian.org/cgi-bin/madison.cgi?table=new',
+    'udd' => 'https://qa.debian.org/cgi-bin/madison.cgi',
+    'new' => 'https://qa.debian.org/cgi-bin/madison.cgi?table=new',
 );
-my $default_url = 'debian';
+my $default_url = 'debian,new';
 if (system('dpkg-vendor', '--is', 'ubuntu') == 0) {
     $default_url = 'ubuntu';
 }
@@ -73,6 +72,7 @@ Display information about PACKAGE(s).
   -g, --greaterorequal       show buildd 'dep-wait pkg >= {highest version}' info
   -G, --greaterthan          show buildd 'dep-wait pkg >> {highest version}' info
   -h, --help                 show this help and exit
+  -r, --regex                treat PACKAGE as a regex [not supported everywhere]
   -s, --suite=SUITE          only show info for this suite
   -S, --source-and-binary    show info for the binary children of source pkgs
   -t, --time                 show projectb snapshot date
@@ -159,10 +159,6 @@ unless (@ARGV) {
     print STDERR "E: need at least one package name as an argument.\n";
     exit 1;
 }
-if ($params->{regex}) {
-    print STDERR "E: rmadison does not support the -r --regex option.\n";
-    exit 1;
-}
 if ($params->{greaterorequal} and $params->{greaterthan}) {
     print STDERR "E: -g/--greaterorequal and -G/--greaterthan are mutually exclusive.\n";
     exit 1;
@@ -179,6 +175,7 @@ push @args, "b=$params->{'binary-type'}" if $params->{'binary-type'};
 push @args, "c=$params->{'component'}" if $params->{'component'};
 push @args, "g" if $params->{'greaterorequal'};
 push @args, "G" if $params->{'greaterthan'};
+push @args, "r" if $params->{'regex'};
 push @args, "s=$params->{'suite'}" if $params->{'suite'};
 push @args, "S" if $params->{'source-and-binary'};
 push @args, "t" if $params->{'time'};
@@ -216,7 +213,7 @@ rmadison -- Remotely query the Debian archive database about packages
 
 B<dak ls> queries the Debian archive database ("projectb") and
 displays which package version is registered per architecture/component/suite.
-The CGI at B<http://qa.debian.org/madison.php> provides that service without
+The CGI at B<https://qa.debian.org/madison.php> provides that service without
 requiring SSH access to ftp-master.debian.org or the mirror on
 ries.debian.org. This script, B<rmadison>, is a command line frontend to
 this CGI.
@@ -253,6 +250,13 @@ show this help and exit
 
 only show info for this suite
 
+=item B<-s>, B<--regex>
+
+treat PACKAGE as a regex
+
+B<Note:> Since B<-r> can easily DoS the database ("-r ."), this option is not
+supported by the CGI on qa.debian.org and most other installations.
+
 =item B<-S>, B<--source-and-binary>
 
 show info for the binary children of source pkgs
@@ -264,12 +268,11 @@ show projectb snapshot and reload time (not supported by all archives)
 =item B<-u>, B<--url=>I<URL>[B<,>I<URL> ...]
 
 use I<URL> for the query. Supported shorthands are
- B<debian> or B<qa> http://qa.debian.org/madison.php
- B<bpo> http://backports.debian.org/cgi-bin/madison.cgi
+ B<debian> or B<qa> https://qa.debian.org/madison.php
  B<debug> http://debug.debian.net/cgi-bin/madison.cgi
  B<ubuntu> http://people.canonical.com/~ubuntu-archive/madison.cgi
- B<udd> http://qa.debian.org/cgi-bin/madison.cgi
- B<new> http://qa.debian.org/cgi-bin/madison.cgi?table=new
+ B<udd> https://qa.debian.org/cgi-bin/madison.cgi
+ B<new> https://qa.debian.org/cgi-bin/madison.cgi?table=new
 
 See the B<RMADISON_URL_MAP_> variable below for a method to add
 new shorthands.
@@ -308,6 +311,7 @@ B<RMADISON_URL_MAP_*> variables.
 =item B<RMADISON_DEFAULT_URL>=I<URL>
 
 Set the default URL to use unless overridden by a command line option.
+For Debian this defaults to debian,new. For Ubuntu this defaults to ubuntu.
 
 =item B<RMADISON_ARCHITECTURE>=I<ARCH>
 
@@ -319,14 +323,10 @@ B<--architecture='*'>.
 
 =head1 NOTES
 
-B<dak ls> also supports B<-r>, B<--regex> to treat I<PACKAGE> as a regex. Since
-that can easily DoS the database ("-r ."), this option is not supported by the
-CGI and rmadison.
-
 B<dak ls> was formerly called B<madison>.
 
 The protocol used by rmadison is fairly simple, the CGI accepts query the
-parameters a, b, c, g, G, s, S, t, and package. The parameter text is passed to
+parameters a, b, c, g, G, r, s, S, t, and package. The parameter text is passed to
 enable plain-text output.
 
 =head1 SEE ALSO
@@ -335,7 +335,7 @@ B<dak>(1), B<madison-lite>(1)
 
 =head1 AUTHOR
 
-rmadison and http://qa.debian.org/madison.php were written by Christoph Berg
+rmadison and https://qa.debian.org/madison.php were written by Christoph Berg
 <myon@debian.org>. dak was written by
 James Troup <james@nocrew.org>, Anthony Towns <ajt@debian.org>, and others.
 
